@@ -32,9 +32,8 @@
 #include <stdarg.h>
 #include <limits.h>
 
-#ifdef HAVE_CRYPT_H
-#include <crypt.h>
-#endif
+#include <usual/tls/tls.h>
+
 #ifdef HAVE_LIBGEN_H
 #include <libgen.h>
 #endif
@@ -58,13 +57,12 @@
  * libc compat functions.
  */
 
-#ifndef HAVE_CRYPT
-static inline char *crypt(const char *p, const char *s) { return NULL; }
-#endif
 #ifndef HAVE_LSTAT
 static inline int lstat(const char *path, struct stat *st) { return stat(path, st); }
 #endif
 
+bool check_unix_peer_name(int fd, const char *username);
+
 void change_user(const char *user);
 
 void change_file_mode(const char *fn, mode_t mode, const char *user, const char *group);
