@@ -62,11 +62,6 @@
 
 #define is_word(c) (isalnum(c) || (c) == '_')
 
-#ifndef isblank
-#define isblank usual_isblank
-static int isblank(int c) { return ((c) == ' ' || (c) == '\t'); }
-#endif
-
 struct Op;
 struct ExecCtx;
 struct GMatch;
@@ -320,11 +315,7 @@ static int op_count_full(struct ParseCtx *ctx, const char **re)
 	}
 
 	/* bad fmt, decide between error codes */
-	for (a = 0; end[a] && a < 5; a++) {
-		if (end[a] == '}')
-			return REG_BADBR;
-	}
-	return REG_EBRACE;
+	return strchr(end, '}') ? REG_BADBR : REG_EBRACE;
 
 done:
 	ctx->last_elem->mincnt = a;
@@ -650,23 +641,14 @@ loop:
 
 escaped:
 	c = *re++;
-	switch (c) {
-	case 0:
-		return REG_EESCAPE;
-	case '.': case '^': case '$':
-	case '*': case '?': case '+':
-	case '(': case ')': case '|':
-	case '{': case '[': case '\\':
-	case '}': case ']':
-		err = op_char(ctx, c);
-		break;
-	case '1': case '2': case '3': case '4': case '5':
-	case '6': case '7': case '8': case '9':
+	if (c == 0)
+		err = REG_EESCAPE;
+	else if (c >= '0' && c <= '9')
 		err = STRICT ? REG_BADPAT : op_bref(ctx, c);
-		break;
-	default:
+	else if (isalpha(c))
 		err = parse_relaxed_escapes(ctx, c);
-	}
+	else
+		err = op_char(ctx, c);
 	goto loop;
 }
 
