@@ -1,4 +1,4 @@
-/* $OpenBSD: tls_util.c,v 1.1 2014/10/31 13:46:17 jsing Exp $ */
+/* $OpenBSD$ */
 /*
  * Copyright (c) 2014 Joel Sing <jsing@openbsd.org>
  * Copyright (c) 2015 Reyk Floeter <reyk@openbsd.org>
@@ -30,7 +30,7 @@
 const char *
 tls_backend_version(void)
 {
-	return SSLeay_version(SSLEAY_VERSION);
+	return OpenSSL_version(OPENSSL_VERSION);
 }
 
 /*
@@ -185,17 +185,21 @@ tls_get_connection_info(struct tls *ctx, char *buf, size_t buflen)
 	const char *proto = "-", *cipher = "-";
 	char dh[64];
 	int used_dh_bits = ctx->used_dh_bits, used_ecdh_nid = ctx->used_ecdh_nid;
+	const SSL_CIPHER *ciph_obj = NULL;
+
+	dh[0] = 0;
 
 	if (conn != NULL) {
 		proto = SSL_get_version(conn);
 		cipher = SSL_get_cipher(conn);
+		ciph_obj = SSL_get_current_cipher(conn);
 
 #ifdef SSL_get_server_tmp_key
 		if (ctx->flags & TLS_CLIENT) {
 			EVP_PKEY *pk = NULL;
 			int ok = SSL_get_server_tmp_key(conn, &pk);
-			int pk_type = EVP_PKEY_id(pk);
-			if (ok && pk) {
+			if (ok) {
+				int pk_type = EVP_PKEY_id(pk);
 				if (pk_type == EVP_PKEY_DH) {
 					DH *dh = EVP_PKEY_get0(pk);
 					used_dh_bits = DH_size(dh) * 8;
@@ -206,16 +210,24 @@ tls_get_connection_info(struct tls *ctx, char *buf, size_t buflen)
 				}
 				EVP_PKEY_free(pk);
 			}
-		}
+		} else
 #endif
+		if (ciph_obj && !used_ecdh_nid && !used_dh_bits) {
+#ifdef SSL_get_shared_curve
+			int kx = SSL_CIPHER_get_kx_nid(ciph_obj);
+			if (kx == NID_kx_ecdhe) {
+				used_ecdh_nid = SSL_get_shared_curve(conn, 0);
+			} else if (kx == NID_kx_dhe) {
+				snprintf(dh, sizeof dh, "/DH=?");
+			}
+#endif
+		}
 	}
 
 	if (used_dh_bits) {
 		snprintf(dh, sizeof dh, "/DH=%d", used_dh_bits);
 	} else if (used_ecdh_nid) {
 		snprintf(dh, sizeof dh, "/ECDH=%s", OBJ_nid2sn(used_ecdh_nid));
-	} else {
-		dh[0] = 0;
 	}
 
 	if (ctx->ocsp_result) {
