@@ -221,6 +221,7 @@ static bool handle_server_work(PgSocket *server, PktHdr *pkt)
 	char state;
 	SBuf *sbuf = &server->sbuf;
 	PgSocket *client = server->link;
+	bool async_response = false;
 
 	Assert(!server->pool->db->admin);
 
@@ -306,6 +307,7 @@ static bool handle_server_work(PgSocket *server, PktHdr *pkt)
 	case 'A':		/* NotificationResponse */
 		idle_tx = server->idle_tx;
 		ready = server->ready;
+		async_response = true;
 		break;
 
 	/* copy mode */
@@ -343,15 +345,28 @@ static bool handle_server_work(PgSocket *server, PktHdr *pkt)
 			return handle_auth_response(client, pkt);
 		} else {
 			sbuf_prepare_send(sbuf, &client->sbuf, pkt->len);
-			if (ready && client->query_start) {
+
+			/* every statement (independent or in a transaction) counts as a query */
+			if ((ready || idle_tx) && client->query_start) {
 				usec_t total;
 				total = get_cached_time() - client->query_start;
 				client->query_start = 0;
 				server->pool->stats.query_time += total;
 				slog_debug(client, "query time: %d us", (int)total);
-			} else if (ready) {
+			} else if ((ready || idle_tx) && !async_response) {
 				slog_warning(client, "FIXME: query end, but query_start == 0");
 			}
+
+			/* statement ending in "idle" ends a transaction */
+			if (ready && client->xact_start) {
+				usec_t total;
+				total = get_cached_time() - client->xact_start;
+				client->xact_start = 0;
+				server->pool->stats.xact_time += total;
+				slog_debug(client, "transaction time: %d us", (int)total);
+			} else if (ready && !async_response) {
+				slog_warning(client, "FIXME: transaction end, but xact_start == 0");
+			}
 		}
 	} else {
 		if (server->state != SV_TESTED)
@@ -370,8 +385,9 @@ static bool handle_connect(PgSocket *server)
 	bool res = false;
 	PgPool *pool = server->pool;
 	char buf[PGADDR_BUF + 32];
+	bool is_unix = pga_is_unix(&server->remote_addr);
 
-	fill_local_addr(server, sbuf_socket(&server->sbuf), pga_is_unix(&server->remote_addr));
+	fill_local_addr(server, sbuf_socket(&server->sbuf), is_unix);
 
 	if (cf_log_connections) {
 		if (pga_is_unix(&server->remote_addr))
@@ -390,7 +406,7 @@ static bool handle_connect(PgSocket *server)
 		disconnect_server(server, false, "sent cancel req");
 	} else {
 		/* proceed with login */
-		if (cf_server_tls_sslmode > SSLMODE_DISABLED) {
+		if (cf_server_tls_sslmode > SSLMODE_DISABLED && !is_unix) {
 			slog_noise(server, "P: SSL request");
 			res = send_sslreq_packet(server);
 			if (res)
